@@ -17,6 +17,7 @@ SECTION(mesh);
 typedef struct _any_t {
 	union {
 		uint32_t as_32;
+		int32_t as_s32;
 		uint16_t as_16;
 		uint8_t as_8;
 	} u;
@@ -96,6 +97,19 @@ static uint32_t _parse_u32(const char *str, _any *ret)
 	return 0;
 }
 
+static uint32_t _parse_s32(const char *str, _any *ret)
+{
+	char *endptr = NULL;
+	long int v = strtol(str, &endptr, 10);
+	if (*endptr != '\0')
+		return 0xffffffff;
+	if (v > 0xff)
+		return 0xffffffff;
+	ret->u.as_s32 = (int32_t)v;
+	return 0;
+}
+
+
 static void _print_u8(struct nlattr *a)
 {
 	printf("%d", nla_get_u8(a));
@@ -116,6 +130,11 @@ static void _print_u16_in_TUs(struct nlattr *a)
 	printf("%d TUs", nla_get_u16(a));
 }
 
+static void _print_u32(struct nlattr *a)
+{
+	printf("%d", nla_get_u32(a));
+}
+
 static void _print_u32_timeout(struct nlattr *a)
 {
 	printf("%u milliseconds", nla_get_u32(a));
@@ -126,6 +145,12 @@ static void _print_u32_in_TUs(struct nlattr *a)
 	printf("%d TUs", nla_get_u32(a));
 }
 
+static void _print_s32_in_dBm(struct nlattr *a)
+{
+	printf("%d dBm", (int32_t) nla_get_u32(a));
+}
+
+
 /* The current mesh parameters */
 const static struct mesh_param_descr _mesh_param_descrs[] =
 {
@@ -174,11 +199,24 @@ const static struct mesh_param_descr _mesh_param_descrs[] =
 	{"mesh_hwmp_rootmode", NL80211_MESHCONF_HWMP_ROOTMODE,
 	_my_nla_put_u8, _parse_u8, _print_u8},
 	{"mesh_hwmp_rann_interval", NL80211_MESHCONF_HWMP_RANN_INTERVAL,
-	_my_nla_put_u16, _parse_u16, _print_u16},
+	_my_nla_put_u16, _parse_u16, _print_u16_in_TUs},
 	{"mesh_gate_announcements", NL80211_MESHCONF_GATE_ANNOUNCEMENTS,
 	_my_nla_put_u8, _parse_u8, _print_u8},
 	{"mesh_fwding", NL80211_MESHCONF_FORWARDING,
 	_my_nla_put_u8, _parse_u8_as_bool, _print_u8},
+	{"mesh_sync_offset_max_neighor",
+	NL80211_MESHCONF_SYNC_OFFSET_MAX_NEIGHBOR,
+	_my_nla_put_u32, _parse_u32, _print_u32},
+	{"mesh_rssi_threshold", NL80211_MESHCONF_RSSI_THRESHOLD,
+	_my_nla_put_u32, _parse_s32, _print_s32_in_dBm},
+	{"mesh_hwmp_active_path_to_root_timeout",
+	NL80211_MESHCONF_HWMP_PATH_TO_ROOT_TIMEOUT,
+	_my_nla_put_u32, _parse_u32, _print_u32_in_TUs},
+	{"mesh_hwmp_root_interval", NL80211_MESHCONF_HWMP_ROOT_INTERVAL,
+	_my_nla_put_u16, _parse_u16, _print_u16_in_TUs},
+	{"mesh_hwmp_confirmation_interval",
+	NL80211_MESHCONF_HWMP_CONFIRMATION_INTERVAL,
+	_my_nla_put_u16, _parse_u16, _print_u16_in_TUs},
 };
 
 static void print_all_mesh_param_descr(void)
@@ -213,7 +251,8 @@ static const struct mesh_param_descr *find_mesh_param(const char *name)
 static int set_interface_meshparam(struct nl80211_state *state,
 				   struct nl_cb *cb,
 				   struct nl_msg *msg,
-				   int argc, char **argv)
+				   int argc, char **argv,
+				   enum id_input id)
 {
 	const struct mesh_param_descr *mdescr;
 	struct nlattr *container;
@@ -319,7 +358,8 @@ static int print_mesh_param_handler(struct nl_msg *msg, void *arg)
 static int get_interface_meshparam(struct nl80211_state *state,
 				   struct nl_cb *cb,
 				   struct nl_msg *msg,
-				   int argc, char **argv)
+				   int argc, char **argv,
+				   enum id_input id)
 {
 	const struct mesh_param_descr *mdescr = NULL;
 
@@ -342,8 +382,10 @@ COMMAND(get, mesh_param, "[<param>]",
 	"Retrieve mesh parameter (run command without any to see available ones).");
 
 static int join_mesh(struct nl80211_state *state, struct nl_cb *cb,
-		     struct nl_msg *msg, int argc, char **argv)
+		     struct nl_msg *msg, int argc, char **argv,
+		     enum id_input id)
 {
+	struct nlattr *container;
 	float rate;
 	char *end;
 
@@ -367,18 +409,40 @@ static int join_mesh(struct nl80211_state *state, struct nl_cb *cb,
 		argc--;
 	}
 
+	container = nla_nest_start(msg, NL80211_ATTR_MESH_SETUP);
+	if (!container)
+		return -ENOBUFS;
+
+	if (argc > 1 && strcmp(argv[0], "vendor_sync") == 0) {
+		argv++;
+		argc--;
+		if (strcmp(argv[0], "on") == 0)
+			NLA_PUT_U8(msg,
+				   NL80211_MESH_SETUP_ENABLE_VENDOR_SYNC, 1);
+		else
+			NLA_PUT_U8(msg,
+				   NL80211_MESH_SETUP_ENABLE_VENDOR_SYNC, 0);
+		argv++;
+		argc--;
+	}
+	/* parse and put other NL80211_ATTR_MESH_SETUP elements here */
+
+	nla_nest_end(msg, container);
+
 	if (!argc)
 		return 0;
-	return set_interface_meshparam(state, cb, msg, argc, argv);
+	return set_interface_meshparam(state, cb, msg, argc, argv, id);
  nla_put_failure:
 	return -ENOBUFS;
 }
-COMMAND(mesh, join, "<mesh ID> [mcast-rate <rate in Mbps>] [<param>=<value>]*",
+COMMAND(mesh, join, "<mesh ID> [mcast-rate <rate in Mbps>] [vendor_sync on|off]"
+	" [<param>=<value>]*",
 	NL80211_CMD_JOIN_MESH, 0, CIB_NETDEV, join_mesh,
 	"Join a mesh with the given mesh ID with mcast-rate and mesh parameters.");
 
 static int leave_mesh(struct nl80211_state *state, struct nl_cb *cb,
-		      struct nl_msg *msg, int argc, char **argv)
+		      struct nl_msg *msg, int argc, char **argv,
+		      enum id_input id)
 {
 	if (argc)
 		return 1;
